package org.klomp.eclipse.team.ui.actions;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.ISynchronizer;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.QualifiedName;

import org.eclipse.swt.widgets.Shell;
import org.eclipse.team.core.TeamException;
import org.eclipse.team.internal.core.target.TargetManager;
import org.eclipse.team.internal.core.target.TargetProvider;

import org.eclipse.ui.IObjectActionDelegate;
import org.eclipse.ui.IWorkbenchPart;

import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.ErrorDialog;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.klomp.eclipse.team.ui.TeamUIPlugin;

public class DisconnectTarget implements IObjectActionDelegate
{
    private final static QualifiedName TARGET_MAPPINGS =
        new QualifiedName("org.eclipse.team.core.target", "mappings");

    /**
     * Key for persistent property to denote the project as having target management
     */
    private final static QualifiedName PROP_KEY = new QualifiedName("org.eclipse.team", "target");

    /**
     * Key to signify the kind of target.  We only have one at the moment
     */
    private final static String BASIC_TARGET_KEY = "basic";

    private List projects = new ArrayList();

    private IAction action;
    private IWorkbenchPart targetPart;

    public DisconnectTarget()
    {
    }

    /**
     * @see IObjectActionDelegate#setActivePart(IAction, IWorkbenchPart)
     */
    public void setActivePart(IAction action, IWorkbenchPart targetPart)
    {
        this.action = action;
        this.targetPart = targetPart;
    }

    /**
     * @see IActionDelegate#run(IAction)
     */
    public void run(IAction action)
    {
        for (Iterator i = projects.iterator(); i.hasNext();)
        {
            IProject project = (IProject) i.next();

            try
            {
                disconnectTarget(project);
            }
            catch (TeamException e)
            {
                ErrorDialog.openError(
                    new Shell(),
                    TeamUIPlugin.getString("Error"),
                    TeamUIPlugin.getString("DisconnectTarget.errorUnmappingProject"),
                    e.getStatus());
            }
            catch (CoreException e)
            {
                ErrorDialog.openError(
                    new Shell(),
                    TeamUIPlugin.getString("Error"),
                    TeamUIPlugin.getString("DisconnectTarget.errorUnmappingProject"),
                    e.getStatus());
            }
        }
    }

    private void disconnectTarget(IProject project) throws CoreException, TeamException
    {
        ISynchronizer s = ResourcesPlugin.getWorkspace().getSynchronizer();
        byte[] mappingBytes = s.getSyncInfo(TARGET_MAPPINGS, project);
        if (mappingBytes != null)
        {
            TargetProvider provider = TargetManager.getProvider(project);
            // De-register IF the provider can be found, otherwise unmap it directly.
            if (provider != null)
            {
                try
                {
                    provider.deregister(project);
                }
                catch (Throwable t)
                {
                    TeamUIPlugin.logError("Deregistering target provider from project; ignored", t);
                }
            }
            s.flushSyncInfo(TARGET_MAPPINGS, project, IResource.DEPTH_ZERO);
        }
        project.setPersistentProperty(PROP_KEY, null); // null arg removes

    }

    private boolean isMapped(IProject project) throws CoreException
    {
        if (project.getPersistentProperty(PROP_KEY) != null)
        {
            return true;
        }
        else
        {
            return false;
        }
    }

    /**
     * @see IActionDelegate#selectionChanged(IAction, ISelection)
     */
    public void selectionChanged(IAction action, ISelection s)
    {
        projects.clear();
        if (!(s instanceof IStructuredSelection))
        {
            return;
        }
        IStructuredSelection selection = (IStructuredSelection) s;
        List selected = selection.toList();
        for (Iterator i = selected.iterator(); i.hasNext();)
        {
            Object o = (Object) i.next();
            if (!(o instanceof IProject))
            {
                continue;
            }
            IProject project = (IProject) o;
            try
            {
                if (isMapped(project))
                {
                    projects.add(project);
                }
            }
            catch (CoreException e)
            {
                ErrorDialog.openError(
                    new Shell(),
                    TeamUIPlugin.getString("Error"),
                    TeamUIPlugin.getString("DisconnectTarget.errorCheckingProject"),
                    e.getStatus());
            }
        }

    }

}
